// Copyright (c) 2019 IoTeX
// This is an alpha (internal) release and is not suitable for production. This source code is provided 'as is' and no
// warranties are given as to title or non-infringement, merchantability or fitness for purpose and, to the extent
// permitted by law, all liability for your use of the code is disclaimed. This source code is governed by Apache
// License 2.0 that can be found in the LICENSE file.

package bc

import (
	"fmt"

	"github.com/spf13/cobra"
)

// bcInfoCmd represents the bc info command
var bcInfoCmd = &cobra.Command{
	Use:   "info",
	Short: "Get current block chain information",
	Args:  cobra.ExactArgs(0),
	RunE: func(cmd *cobra.Command, args []string) error {
		cmd.SilenceUsage = true
		output, err := bcInfo()
		if err == nil {
			fmt.Println(output)
		}
		return err
	},
}

// bcInfo get current information of block chain from server
func bcInfo() (string, error) {
	chainMeta, err := GetChainMeta()
	if err != nil {
		return "", err
	}
	return fmt.Sprintf("height:%d  numActions:%d  tps:%3f\nepochNum:%d  epochStartHeight:%d"+
		"  gravityChainStartHeight:%d", chainMeta.Height, chainMeta.NumActions, chainMeta.TpsFloat,
		chainMeta.Epoch.Num, chainMeta.Epoch.Height, chainMeta.Epoch.GravityChainStartHeight), nil
}
